"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerController = exports.createGlobalScopeMiddleware = exports.createLicenseMiddleware = void 0;
const typedi_1 = require("typedi");
const express_1 = require("express");
const express_rate_limit_1 = require("express-rate-limit");
const n8n_workflow_1 = require("n8n-workflow");
const auth_service_1 = require("../auth/auth.service");
const config_1 = __importDefault(require("../config"));
const constants_1 = require("../constants");
const License_1 = require("../License");
const ResponseHelper_1 = require("../ResponseHelper");
const constants_2 = require("./constants");
const throttle = (0, express_rate_limit_1.rateLimit)({
    windowMs: 5 * 60 * 1000,
    limit: 5,
    message: { message: 'Too many requests' },
});
const createLicenseMiddleware = (features) => (_req, res, next) => {
    if (features.length === 0) {
        return next();
    }
    const licenseService = typedi_1.Container.get(License_1.License);
    const hasAllFeatures = features.every((feature) => licenseService.isFeatureEnabled(feature));
    if (!hasAllFeatures) {
        return res
            .status(403)
            .json({ status: 'error', message: 'Plan lacks license for this feature' });
    }
    return next();
};
exports.createLicenseMiddleware = createLicenseMiddleware;
const createGlobalScopeMiddleware = (scopes) => async ({ user }, res, next) => {
    if (scopes.length === 0) {
        return next();
    }
    if (!user)
        return res.status(401).json({ status: 'error', message: 'Unauthorized' });
    const hasScopes = user.hasGlobalScope(scopes);
    if (!hasScopes) {
        return res.status(403).json({ status: 'error', message: 'Unauthorized' });
    }
    return next();
};
exports.createGlobalScopeMiddleware = createGlobalScopeMiddleware;
const registerController = (app, controllerClass) => {
    var _a;
    const controller = typedi_1.Container.get(controllerClass);
    const controllerBasePath = Reflect.getMetadata(constants_2.CONTROLLER_BASE_PATH, controllerClass);
    if (!controllerBasePath)
        throw new n8n_workflow_1.ApplicationError('Controller is missing the RestController decorator', {
            extra: { controllerName: controllerClass.name },
        });
    const routes = Reflect.getMetadata(constants_2.CONTROLLER_ROUTES, controllerClass);
    const licenseFeatures = Reflect.getMetadata(constants_2.CONTROLLER_LICENSE_FEATURES, controllerClass);
    const requiredScopes = Reflect.getMetadata(constants_2.CONTROLLER_REQUIRED_SCOPES, controllerClass);
    if (routes.length > 0) {
        const router = (0, express_1.Router)({ mergeParams: true });
        const restBasePath = config_1.default.getEnv('endpoints.rest');
        const prefix = `/${[restBasePath, controllerBasePath].join('/')}`
            .replace(/\/+/g, '/')
            .replace(/\/$/, '');
        const controllerMiddlewares = ((_a = Reflect.getMetadata(constants_2.CONTROLLER_MIDDLEWARES, controllerClass)) !== null && _a !== void 0 ? _a : []).map(({ handlerName }) => controller[handlerName].bind(controller));
        const authService = typedi_1.Container.get(auth_service_1.AuthService);
        routes.forEach(({ method, path, middlewares: routeMiddlewares, handlerName, usesTemplates, skipAuth, rateLimit, }) => {
            var _a, _b;
            const features = (_a = licenseFeatures === null || licenseFeatures === void 0 ? void 0 : licenseFeatures[handlerName]) !== null && _a !== void 0 ? _a : licenseFeatures === null || licenseFeatures === void 0 ? void 0 : licenseFeatures['*'];
            const scopes = (_b = requiredScopes === null || requiredScopes === void 0 ? void 0 : requiredScopes[handlerName]) !== null && _b !== void 0 ? _b : requiredScopes === null || requiredScopes === void 0 ? void 0 : requiredScopes['*'];
            const handler = async (req, res) => await controller[handlerName](req, res);
            router[method](path, ...(!constants_1.inTest && !constants_1.inE2ETests && rateLimit ? [throttle] : []), ...(skipAuth ? [] : [authService.authMiddleware]), ...(features ? [(0, exports.createLicenseMiddleware)(features)] : []), ...(scopes ? [(0, exports.createGlobalScopeMiddleware)(scopes)] : []), ...controllerMiddlewares, ...routeMiddlewares, usesTemplates ? handler : (0, ResponseHelper_1.send)(handler));
        });
        app.use(prefix, router);
    }
};
exports.registerController = registerController;
//# sourceMappingURL=registerController.js.map